//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	CoordinateFrame	--- a class representing a cartesian coordinate frame
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		UL, 1999-03-08: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers, Mon Mar  8 12:27:21 MET 1999
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /usr/local/cvs/sasdev/sas/packages/caloalutils/src/CoordinateFrame.h,v 1.3 2002/05/24 12:46:05 ulammers Exp $
//

#ifndef COORDINATE_FRAME_H
#define COORDINATE_FRAME_H

#include <cassert>
#include "Celestial.h"

class CoordinateFrameSkeleton {
public:
	CoordinateFrameSkeleton(double ox = 0., double oy = 0., double oz = 0.,
							double xx = 1., double xy = 0., double xz = 0.,
							double yx = 0., double yy = 1., double yz = 0.)
		: _origin(ox, oy, oz), _plusX(xx, xy, xz), _plusY(yx, yy, yz)
		{ _plusX.normalize(); _plusY.normalize();
				assert(_plusX * _plusY <= 1.e-1); }
	CoordinateFrameSkeleton(CartesianVector &origin, CartesianVector &plusX,
							CartesianVector &plusY): 
		_origin(origin), _plusX(plusX), _plusY(plusY)
		{ _plusX.normalize(); _plusY.normalize();
				assert(_plusX * _plusY <= 1.e-1); }

	const CartesianVector &origin() const { return _origin; }
	const CartesianVector &plusX() const { return _plusX; }
	const CartesianVector &plusY() const { return _plusY; }

	~CoordinateFrameSkeleton() {}

protected:
	CartesianVector	_origin,
					_plusX,
					_plusY;
};

class CoordinateFrame : public CoordinateFrameSkeleton {
public:
	CoordinateFrame(double ox = 0., double oy = 0., double oz = 0.,
					double xx = 1., double xy = 0., double xz = 0.,
					double yx = 0., double yy = 1., double yz = 0.) :
		CoordinateFrameSkeleton(ox, oy, oz, xx, xy, xz, yx, yy, yz)
		{ complete(); }
	CoordinateFrame(CartesianVector &origin, CartesianVector &plusX,
					CartesianVector &plusY) :
		CoordinateFrameSkeleton(origin, plusX, plusY)
		{ complete(); }

	const CartesianVector& plusZ() const { return _plusZ; }

	CartesianVector project(const CartesianVector &v) const;
	CartesianVector project(double x, double y, double z) const
		{ return project(CartesianVector(x, y, z)); }

	~CoordinateFrame() {}

private:
	void complete();
	CartesianVector	_plusZ;
};

#endif
