//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Interpolator.h	--- polynomial interpolation
//
//	DESCRIPTION:
//		Polynomial interpolation
//
//	CHANGE HISTORY:
//		UL, 1998-08-11: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers, Tue Aug 11 15:55:34 MET DST 1998
//
//	REFERENCES:
//		[1]: Numerical Recipes in C, Second Edition, W.H. Press et. al.
//
//	$Header: /data/xmm02/cvsadmin/soc_repos/sas/packages/caloalutils/src/Interpolator.h,v 1.2 2002/02/12 15:40:54 ulammers Exp $
//

#ifndef INTERPOLATOR_H
#define INTERPOLATOR_H

#include "errstr.h"

template<unsigned N>
class Interpolator {
public:
	static void doPoly4(double xp[4], double yp[4][N], double x, double y[N]) {
		//
		//	PURPOSE:
		//		Interpolation with a 4th order polynomials
		//
		//	ARGUMENTS:
		//		I:	xp[4]	: vector with coordinates of known points
		//			yp[4][N]: N sets of values of the polynomials at the <xp>'s
		//			x		: coordinate at which the polynomials are to be
		//					  evaluated
		//		O:	y[N]	: values of the polynomials at point <x>
		//
		double      ttx[4], txty[4][3];

		for (unsigned i=0; i<4; ++i) {
			unsigned    k;

			ttx[i] = x - xp[i];
			k = 0;
			for (unsigned j=0; j<3; ++j) {
				txty[i][j] = xp[i] - xp[i!=j ? k : ++k];
				if (txty[i][j] == 0.)
					errstr<<"Two points are identical - cannot construct "
							"a polynomial"
						 <<Sas::error("IllegalValues");
				++k;
			}
		}

		for (unsigned i=0; i<N; ++i)
			//	straight Lagrange-formula implementation
			y[i] =
				(ttx[1]*ttx[2]*ttx[3])/(txty[0][0]*txty[0][1]*txty[0][2]) *
					yp[0][i] +
				(ttx[0]*ttx[2]*ttx[3])/(txty[1][0]*txty[1][1]*txty[1][2]) *
					yp[1][i] +
				(ttx[0]*ttx[1]*ttx[3])/(txty[2][0]*txty[2][1]*txty[2][2]) *
					yp[2][i] +
				(ttx[0]*ttx[1]*ttx[2])/(txty[3][0]*txty[3][1]*txty[3][2]) *
					yp[3][i];
	} // doPoly4()
};

#endif



